# -*- coding: utf-8 -*-
import datetime
import os
import pkginfo
import sys
import time

on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
pkg_info = pkginfo.Develop(os.path.join(os.path.dirname(__file__), '..'))
build_date = datetime.datetime.utcfromtimestamp(int(os.environ.get('SOURCE_DATE_EPOCH', time.time())))

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.intersphinx'
    ]

intersphinx_mapping = {
    'http://docs.python.org': None,
    'http://django.readthedocs.org/en/latest/': None,
}

# General
source_suffix = '.txt'
master_doc = 'index'
project = pkg_info.name
copyright = '2008-2015 Simplistix Ltd, 2016-%s Chris Withers' % build_date.year
version = release = pkg_info.version
exclude_trees = ['_build']
exclude_patterns = ['description.txt']
pygments_style = 'sphinx'

# Options for HTML output
if on_rtd:
    html_theme = 'default'
else:
    html_theme = 'classic'
htmlhelp_basename = project+'doc'

# Options for LaTeX output
latex_documents = [
    ('index', project+'.tex', project+u' Documentation',
     'Simplistix Ltd', 'manual'),
]



from __future__ import absolute_import

# Copyright (c) 2010-2013 Simplistix Ltd
#
# See license.txt for more details.
import os
import re
import textwrap

from manuel import Manuel
from testfixtures import diff

FILEBLOCK_START = re.compile(r'^\.\.\s*topic::?\s*(.+)\b', re.MULTILINE)
FILEBLOCK_END = re.compile(r'(\n\Z|\n(?=\S))')
CLASS = re.compile(r'\s+:class:\s*(read|write)-file')


class FileBlock(object):
    def __init__(self, path, content, action):
        self.path, self.content, self.action = path, content, action


class FileResult(object):
    passed = True
    expected = None
    actual = None


class Files(Manuel):
    """
    A `Manuel <http://packages.python.org/manuel/>`__ plugin that
    parses certain ReST sections to read and write files in the
    configured :class:`TempDirectory`.

    :param name: This is the name of the :class:`TempDirectory` to use
                 in the Manual global namespace (ie: `globs`).

    """
    def __init__(self, name):
        self.name = name
        Manuel.__init__(self,
                        parsers=[self.parse],
                        evaluaters=[self.evaluate],
                        formatters=[self.format])

    def parse(self, document):
        for region in document.find_regions(FILEBLOCK_START, FILEBLOCK_END):
            lines = region.source.splitlines()
            class_ = CLASS.match(lines[1])
            if not class_:
                continue
            index = 3
            if lines[index].strip() == '::':
                index += 1
            source = textwrap.dedent('\n'.join(lines[index:])).lstrip()
            if source[-1] != '\n':
                source += '\n'
            region.parsed = FileBlock(
                region.start_match.group(1),
                source,
                class_.group(1)
                )
            document.claim_region(region)

    def evaluate(self, region, document, globs):
        if not isinstance(region.parsed, FileBlock):
            return
        block = region.parsed
        dir = globs[self.name]
        result = region.evaluated = FileResult()
        if block.action == 'read':
            actual = dir.read(block.path, 'ascii').replace(os.linesep, '\n')
            if actual != block.content:
                result.passed = False
                result.path = block.path
                result.expected = block.content
                result.actual = actual
        if block.action == 'write':
            dir.write(block.path, block.content, 'ascii')

    def format(self, document):
        for region in document:
            result = region.evaluated
            if not isinstance(result, FileResult):
                continue
            if not result.passed:
                region.formatted = diff(
                    result.expected,
                    result.actual,
                    'File "%s", line %i:' % (document.location, region.lineno),
                    'Reading from "%s":' % result.path
                )

        return



# Copyright (c) 2009-2013 Simplistix Ltd
#
# See license.txt for more details.

from doctest import REPORT_NDIFF, ELLIPSIS
from glob import glob
from manuel import doctest, capture
from manuel.testing import TestSuite
from nose.plugins.skip import SkipTest
from os.path import dirname, join, pardir

from . import compat

tests = glob(join(join(dirname(__file__), pardir, pardir), 'docs', '*.txt'))

if not tests:
    # tox can't find docs and installing an sdist doesn't install the docs
    raise SkipTest('No docs found to test')  # pragma: no cover


def test_suite():
    m = doctest.Manuel(optionflags=REPORT_NDIFF | ELLIPSIS,
                       checker=compat.DocTestChecker())
    m += compat.Manuel()
    m += capture.Manuel()
    return TestSuite(m, *tests)



# Copyright (c) 2008 Simplistix Ltd
# See license.txt for license details.

from unittest import TestCase, TestSuite, makeSuite

from testfixtures import generator
from types import GeneratorType


class TestG(TestCase):

    def test_example(self):
        g = generator(1, 2, 3)
        self.failUnless(isinstance(g, GeneratorType))
        self.assertEqual(tuple(g), (1, 2, 3))

    def test_from_sequence(self):
        s = (1, 2, 3)
        g = generator(*s)
        self.failUnless(isinstance(g, GeneratorType))
        self.assertEqual(tuple(g), (1, 2, 3))



from __future__ import print_function
# Copyright (c) 2008-2013 Simplistix Ltd
# See license.txt for license details.

from doctest import DocTestSuite
from mock import Mock
from testfixtures import Replacer, LogCapture, compare
from unittest import TestSuite, TestCase, makeSuite

from logging import getLogger

from warnings import catch_warnings

root = getLogger()
one = getLogger('one')
two = getLogger('two')
child = getLogger('one.child')


class DemoLogCapture:

    def test_simple(self):  # pragma: no branch
        """
        >>> root.info('some logging')
        >>> print(log_capture)
        root INFO
          some logging
        >>> log_capture.clear()
        >>> print(log_capture)
        No logging captured
        >>> root.info('some more logging')
        >>> print(log_capture)
        root INFO
          some more logging
        """


class TestLogCapture:

    def test_simple(self):  # pragma: no branch
        """
        >>> root.info('before')
        >>> l = LogCapture()
        >>> root.info('during')
        >>> l.uninstall()
        >>> root.info('after')
        >>> print(l)
        root INFO
          during
        """

    def test_specific_logger(self):  # pragma: no branch
        """
        >>> l = LogCapture('one')
        >>> root.info('1')
        >>> one.info('2')
        >>> two.info('3')
        >>> child.info('4')
        >>> l.uninstall()
        >>> print(l)
        one INFO
          2
        one.child INFO
          4
        """

    def test_multiple_loggers(self):  # pragma: no branch
        """
        >>> l = LogCapture(('one.child','two'))
        >>> root.info('1')
        >>> one.info('2')
        >>> two.info('3')
        >>> child.info('4')
        >>> l.uninstall()
        >>> print(l)
        two INFO
          3
        one.child INFO
          4
        """

    def test_simple_manual_install(self):  # pragma: no branch
        """
        >>> l = LogCapture(install=False)
        >>> root.info('before')
        >>> l.install()
        >>> root.info('during')
        >>> l.uninstall()
        >>> root.info('after')
        >>> print(l)
        root INFO
          during
        """

    def test_uninstall(self):  # pragma: no branch
        """
        Lets start off with a couple of loggers:

        >>> root = getLogger()
        >>> child = getLogger('child')

        Lets also record the handlers for these loggers before
        we start the test:

        >>> before_root = root.handlers[:]
        >>> before_child = child.handlers[:]

        Lets also record the levels for the loggers:

        >>> old_root_level=root.level
        >>> old_child_level=child.level

        Now the test:

        >>> try:
        ...    root.setLevel(49)
        ...    child.setLevel(69)
        ...
        ...    l1 = LogCapture()
        ...    l2 = LogCapture('child')
        ...
        ...    root = getLogger()
        ...    root.info('1')
        ...    print('root level during test:',root.level)
        ...    child = getLogger('child')
        ...    print('child level during test:',child.level)
        ...    child.info('2')
        ...    print('l1 contents:')
        ...    print(l1)
        ...    print('l2 contents:')
        ...    print(l2)
        ...
        ...    l2.uninstall()
        ...    l1.uninstall()
        ...
        ...    print('root level after test:',root.level)
        ...    print('child level after test:',child.level)
        ...
        ... finally:
        ...    root.setLevel(old_root_level)
        ...    child.setLevel(old_child_level)
        root level during test: 1
        child level during test: 1
        l1 contents:
        root INFO
          1
        child INFO
          2
        l2 contents:
        child INFO
          2
        root level after test: 49
        child level after test: 69

        Now we check the handlers are as they were before
        the test:

        >>> root.handlers == before_root
        True
        >>> child.handlers == before_child
        True
        """

    def test_uninstall_all(self):  # pragma: no branch
        """
        For this test, it's better if we don't have any
        LogCaptures around when we start:

        >>> log_capture.uninstall()

        If you create several LogCaptures during a doctest,
        it can create clutter to uninstall them all.
        If this is the case, use the classmethod
        LogCapture.uninstall_all() as a tearDown function
        to remove them all:

        >>> before_handlers_root = root.handlers[:]
        >>> before_handlers_child = child.handlers[:]

        >>> l1 = LogCapture()
        >>> l2 = LogCapture('one.child')

        We can see that the LogCaptures have changed the
        handlers, removing existing ones and installing
        their own:

        >>> len(root.handlers)
        1
        >>> root.handlers==before_handlers_root
        False
        >>> len(child.handlers)
        1
        >>> child.handlers==before_handlers_child
        False

        Now we show the function in action:

        >>> LogCapture.uninstall_all()

        ...and we can see the handlers are back as
        they were beefore:

        >>> before_handlers_root == root.handlers
        True
        >>> before_handlers_child == child.handlers
        True
        """

    def test_two_logcaptures_on_same_logger(self):  # pragma: no branch
        """
        If you create more than one LogCaptures on a single
        logger, the 2nd one installed will stop the first
        one working!

        >>> l1 = LogCapture()
        >>> root.info('1st message')
        >>> print(l1)
        root INFO
          1st message
        >>> l2 = LogCapture()
        >>> root.info('2nd message')

        So, l1 missed this message:

        >>> print(l1)
        root INFO
          1st message

        ...because l2 kicked it out and recorded the message:

        >>> print(l2)
        root INFO
          2nd message
        """

    def test_uninstall_more_than_once(self):  # pragma: no branch
        """
        For this test, it's better if we don't have any
        LogCaptures around when we start:

        >>> log_capture.uninstall()

        There's no problem with uninstalling a LogCapture
        more than once:

        >>> old_level = root.level
        >>> try:
        ...    root.setLevel(49)
        ...
        ...    l = LogCapture()
        ...
        ...    print('root level during test:',root.level)
        ...
        ...    l.uninstall()
        ...
        ...    print('root level after uninstall:',root.level)
        ...
        ...    root.setLevel(69)
        ...
        ...    l.uninstall()
        ...
        ...    print('root level after another uninstall:',root.level)
        ...
        ... finally:
        ...    root.setLevel(old_level)
        root level during test: 1
        root level after uninstall: 49
        root level after another uninstall: 69

        And even when loggers have been uninstalled, there's
        no problem having uninstall_all as a backstop:

        >>> log_capture.uninstall_all()
        """

    def test_with_statement(self):  # pragma: no branch
        """
        >>> root.info('before')
        >>> with LogCapture() as l:
        ...   root.info('during')
        >>> root.info('after')
        >>> print(l)
        root INFO
          during
        """


class LogCaptureTests(TestCase):

    def test_remove_existing_handlers(self):
        logger = getLogger()
        # get original handlers
        original_handlers = logger.handlers
        # put in a stub which will blow up if used
        try:
            logger.handlers = start = [object()]

            with LogCapture() as l:
                logger.info('during')  # pragma: no branch

            l.check(('root', 'INFO', 'during'))

            compare(logger.handlers, start)

        finally:
            # only executed if the test fails
            logger.handlers = original_handlers

    def test_atexit(self):
        # http://bugs.python.org/issue25532
        from mock import call

        m = Mock()
        with Replacer() as r:
            # make sure the marker is false, other tests will
            # probably have set it
            r.replace('testfixtures.LogCapture.atexit_setup', False)
            r.replace('atexit.register', m.register)

            l = LogCapture()

            expected = [call.register(l.atexit)]

            compare(expected, m.mock_calls)

            with catch_warnings(record=True) as w:
                l.atexit()
                self.assertTrue(len(w), 1)
                compare(str(w[0].message), (  # pragma: no branch
                    "LogCapture instances not uninstalled by shutdown, "
                    "loggers captured:\n"
                    "(None,)"
                    ))

            l.uninstall()

            compare(set(), LogCapture.instances)

            # check re-running has no ill effects
            l.atexit()

    def test_numeric_log_level(self):
        with LogCapture() as log:
            getLogger().log(42, 'running in the family')

        log.check(('root', 'Level 42', 'running in the family'))

    def test_enable_disabled_logger(self):
        logger = getLogger('disabled')
        logger.disabled = True
        with LogCapture('disabled') as log:
            logger.info('a log message')
        log.check(('disabled', 'INFO', 'a log message'))
        compare(logger.disabled, True)

    def test_no_propogate(self):
        logger = getLogger('child')
        # paranoid check
        compare(logger.propagate, True)
        with LogCapture() as global_log:
            with LogCapture('child', propagate=False) as child_log:
                logger.info('a log message')
                child_log.check(('child', 'INFO', 'a log message'))
        global_log.check()
        compare(logger.propagate, True)

# using a set up and teardown function
# gets rid of the need for the imports in
# doc tests


def setUp(test):
    test.globs['log_capture'] = LogCapture()


def tearDown(test):
    test.globs['log_capture'].uninstall_all()


def test_suite():
    return TestSuite((
        DocTestSuite(setUp=setUp, tearDown=tearDown),
        makeSuite(LogCaptureTests),
        ))



# Copyright (c) 2010-2013 Simplistix Ltd
#
# See license.txt for more details.

import re

from manuel import Document, Region, RegionContainer, Manuel
from mock import Mock
from testfixtures import compare, Comparison as C, TempDirectory
from testfixtures.manuel import Files, FileBlock, FileResult
from unittest import TestCase


class TestContainer(RegionContainer):
    def __init__(self, attr, *blocks):
        self.regions = []
        for block in blocks:
            region = Region(0, ' ')
            setattr(region, attr, block)
            self.regions.append(region)


class TestManuel(TestCase):

    def tearDown(self):
        TempDirectory.cleanup_all()

    def test_multiple_files(self):
        d = Document("""

.. topic:: file.txt
 :class: write-file

  line 1

  line 2
  line 3

.. topic:: file2.txt
 :class: read-file


  line 4

  line 5
  line 6

""")
        d.parse_with(Files('td'))
        compare([
                None,
                C(FileBlock,
                  path='file.txt',
                  content="line 1\n\nline 2\nline 3\n",
                  action='write'),
                C(FileBlock,
                  path='file2.txt',
                  content='line 4\n\nline 5\nline 6\n',
                  action='read'),
                ], [r.parsed for r in d])

    def test_ignore_literal_blocking(self):
        d = Document("""
.. topic:: file.txt
 :class: write-file

  ::

    line 1

    line 2
    line 3
""")
        d.parse_with(Files('td'))
        compare([
                None,
                C(FileBlock,
                  path='file.txt',
                  content="line 1\n\nline 2\nline 3\n",
                  action='write'),
                ], [r.parsed for r in d])

    def test_file_followed_by_text(self):
        d = Document("""

.. topic:: file.txt
 :class: write-file

  .. code-block:: python

  print "hello"
  out = 'there'

  foo = 'bar'

This is just some normal text!
""")
        d.parse_with(Files('td'))
        compare([
                None,
                C(FileBlock,
                  path='file.txt',
                  content='.. code-block:: python\n\nprint "hello"'
                          '\nout = \'there\'\n\nfoo = \'bar\'\n',
                  action='write'),
                None,
                ], [r.parsed for r in d])

    def test_red_herring(self):
        d = Document("""
.. topic:: file.txt
 :class: not-a-file

  print "hello"
  out = 'there'

""")
        d.parse_with(Files('td'))
        compare([r.parsed for r in d], [None])

    def test_no_class(self):
        d = Document("""
.. topic:: file.txt

  print "hello"
  out = 'there'

""")
        d.parse_with(Files('td'))
        compare([r.parsed for r in d], [None])

    def test_unclaimed_works(self):
        # a test manuel
        CLASS = re.compile(r'^\s+:class:', re.MULTILINE)

        class Block(object):
            def __init__(self, source):
                self.source = source

        def find_class_blocks(document):
            for region in document.find_regions(CLASS):
                region.parsed = Block(region.source)
                document.claim_region(region)

        def Test():
            return Manuel(parsers=[find_class_blocks])

        # now our test
        d = Document("""

.. topic:: something-else
 :class: not-a-file
  line 1
  line 2
  line 3

""")
        d.parse_with(Files('td')+Test())
        # now check FileBlock didn't mask class block
        compare([
                None,
                C(Block,
                  source=' :class:\n'),
                None,
                ], [r.parsed for r in d])

    def test_evaluate_non_fileblock(self):
        m = Mock()
        d = TestContainer('parsed', m)
        d.evaluate_with(Files('td'), globs={})
        compare([None], [r.evaluated for r in d])
        compare(m.call_args_list, [])
        compare(m.method_calls, [])

    def test_evaluate_read_same(self):
        dir = TempDirectory()
        dir.write('foo', b'content')
        d = TestContainer('parsed', FileBlock('foo', 'content', 'read'))
        d.evaluate_with(Files('td'), globs={'td': dir})
        compare([C(FileResult,
                   passed=True,
                   expected=None,
                   actual=None)],
                [r.evaluated for r in d])

    def test_evaluate_read_difference(self):
        dir = TempDirectory()
        dir.write('foo', b'actual')
        d = TestContainer('parsed', FileBlock('foo', 'expected', 'read'))
        d.evaluate_with(Files('td'), globs={'td': dir})
        compare([C(FileResult,
                   passed=False,
                   path='foo',
                   expected='expected',
                   actual='actual')],
                [r.evaluated for r in d])

    def test_evaulate_write(self):
        dir = TempDirectory()
        d = TestContainer('parsed', FileBlock('foo', 'content', 'write'))
        d.evaluate_with(Files('td'), globs={'td': dir})
        compare([C(FileResult,
                   passed=True,
                   expected=None,
                   actual=None)],
                [r.evaluated for r in d])
        dir.compare(['foo'])
        compare(dir.read('foo', 'ascii'), 'content')

    def test_formatter_non_fileblock(self):
        d = TestContainer('evaluated', object)
        d.format_with(Files('td'))
        compare(d.formatted(), '')

    def test_formatter_passed(self):
        d = TestContainer('evaluated', FileResult())
        d.format_with(Files('td'))
        compare(d.formatted(), '')

    def test_formatter_failed(self):
        r = FileResult()
        r.passed = False
        r.path = '/foo/bar'
        r.expected = 'same\nexpected\n'
        r.actual = 'same\nactual\n'
        d = TestContainer('evaluated', r)
        d.format_with(Files('td'))
        compare('--- File "<memory>", line 0:\n'
                '+++ Reading from "/foo/bar":\n'
                '@@ -1,3 +1,3 @@\n'
                ' same\n'
                '-expected\n'
                '+actual\n ',
                d.formatted()
                )



from  __future__ import absolute_import
from functools import partial

from django.db.models import Model
from django.forms import model_to_dict

from .comparison import _compare_mapping, register
from . import compare as base_compare


def compare_model(x, y, context):
    ignore_fields = context.get_option('ignore_fields', set())
    args = []
    for obj in x, y:
        args.append({k: v for (k, v) in model_to_dict(obj).items()
                     if k not in ignore_fields})
    args.append(context)
    args.append(x)
    return _compare_mapping(*args)

register(Model, compare_model)


compare = partial(base_compare, ignore_eq=True)



# Copyright (c) 2010-2012 Simplistix Ltd
#
# See license.txt for more details.
from os.path import dirname
path_to_your_docs = dirname(__file__)

from glob import glob
from manuel import doctest, capture
from manuel.testing import TestSuite
from os.path import join
from testfixtures import TempDirectory
from testfixtures.manuel import Files

from . import compat


def setUp(test):
    test.globs['tempdir'] = TempDirectory()


def tearDown(test):
    test.globs['tempdir'].cleanup()


def test_suite():
    m = doctest.Manuel()
    m += compat.Manuel()
    m += capture.Manuel()
    m += Files('tempdir')
    return TestSuite(
        m,
        setUp=setUp,
        tearDown=tearDown,
        *glob(join(path_to_your_docs, '*.txt'))
        )



# Copyright (c) 2008-2013 Simplistix Ltd
# See license.txt for license details.

from doctest import DocTestSuite, REPORT_NDIFF, ELLIPSIS
from testfixtures import Replacer
from unittest import TestSuite


class TestReplacer:

    def test_function(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1
        >>> sample1.z()
        'original z'

        >>> def test_z():
        ...   return 'replacement z'

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.z',test_z)

        >>> sample1.z()
        'replacement z'

        >>> r.restore()

        >>> sample1.z()
        'original z'
        """

    def test_class(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1
        >>> sample1.X()
        <testfixtures.tests.sample1.X ...>

        >>> class XReplacement(sample1.X): pass

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.X', XReplacement)

        >>> sample1.X()
        <testfixtures.tests.test_replacer.XReplacement ...>
        >>> sample1.X().y()
        'original y'

        >>> r.restore()

        >>> sample1.X()
        <testfixtures.tests.sample1.X ...>
        """

    def test_method(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1
        >>> sample1.X().y()
        'original y'

        >>> def test_y(self):
        ...   return 'replacement y'

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.X.y',test_y)

        >>> sample1.X().y()[:38]
        'replacement y'

        >>> r.restore()

        >>> sample1.X().y()
        'original y'
        """

    def test_class_method(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1
        >>> sample1.X.aMethod()
        <class ...testfixtures.tests.sample1.X...>

        >>> def rMethod(cls):
        ...   return (cls,1)

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.X.aMethod',rMethod)

        >>> sample1.X.aMethod()
        (<class ...testfixtures.tests.sample1.X...>, 1)

        >>> r.restore()

        >>> sample1.X.aMethod()
        <class ...testfixtures.tests.sample1.X...>
        """

    def test_multiple_replace(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1
        >>> sample1.z()
        'original z'
        >>> sample1.X().y()
        'original y'

        >>> def test_y(self):
        ...   return repr(self)
        >>> def test_z():
        ...   return 'replacement z'

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.z',test_z)
        >>> r.replace('testfixtures.tests.sample1.X.y',test_y)

        >>> sample1.z()
        'replacement z'
        >>> sample1.X().y()
        '<testfixtures.tests.sample1.X ...>'

        >>> r.restore()

        >>> sample1.z()
        'original z'
        >>> sample1.X().y()
        'original y'
        """

    def test_gotcha(self):  # pragma: no branch
        """
        Just because you replace an object in one context:

        >>> from testfixtures.tests import sample1
        >>> from testfixtures.tests import sample2
        >>> sample1.z()
        'original z'

        >>> def test_z():
        ...   return 'replacement z'

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.z',test_z)

        >>> sample1.z()
        'replacement z'

        Doesn't meant that it's replaced in all contexts:

        >>> sample2.z()
        'original z'

        >>> r.restore()
        """

    def test_remove_called_twice(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1

        >>> def test_z():
        ...   return 'replacement z'

        >>> r = Replacer()
        >>> r.replace('testfixtures.tests.sample1.z',test_z)

        >>> r.restore()
        >>> sample1.z()
        'original z'

        >>> r.restore()
        >>> sample1.z()
        'original z'
        """

    def test_with_statement(self):  # pragma: no branch
        """
        >>> from testfixtures.tests import sample1
        >>> sample1.z()
        'original z'

        >>> def test_z():
        ...   return 'replacement z'

        >>> with Replacer() as r:
        ...   r.replace('testfixtures.tests.sample1.z',test_z)
        ...   sample1.z()
        'replacement z'

        >>> sample1.z()
        'original z'
        """

    def test_not_there(self):  # pragma: no branch
        """
        >>> def test_bad():
        ...   return 'moo'

        >>> with Replacer() as r:
        ...   r.replace('testfixtures.tests.sample1.bad',test_bad)
        Traceback (most recent call last):
        ...
        AttributeError: Original 'bad' not found
        """


def test_suite():
    return DocTestSuite(optionflags=REPORT_NDIFF | ELLIPSIS)



# Copyright (c) 2008-2014 Simplistix Ltd, 2015-2016 Chris Withers
# See license.txt for license details.

import os

from setuptools import setup, find_packages

name = 'testfixtures'
base_dir = os.path.dirname(__file__)

setup(
    name=name,
    version=open(os.path.join(base_dir, name, 'version.txt')).read().strip(),
    author='Chris Withers',
    author_email='chris@simplistix.co.uk',
    license='MIT',
    description=("A collection of helpers and mock objects "
                 "for unit tests and doc tests."),
    long_description=open(os.path.join(base_dir,
                                       'docs',
                                       'description.txt')).read(),
    url='https://github.com/Simplistix/testfixtures',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    extras_require=dict(
        test=['nose', 'nose-fixes', 'nose-cov', 'mock', 'manuel', 'coveralls',
              'zope.component', 'django'],
        build=['sphinx', 'pkginfo', 'setuptools-git', 'wheel', 'twine']
    )
)



import django
import os
from unittest import TestCase

from testfixtures.compat import PY3
from ... import compare
from ...django import compare as django_compare
from ..test_compare import CompareHelper

os.environ['DJANGO_SETTINGS_MODULE'] = 'testfixtures.tests.test_django.settings'
django.setup()

from .models import SampleModel


class CompareTests(CompareHelper, TestCase):

    def test_simple_same(self):
        django_compare(SampleModel(id=1), SampleModel(id=1))

    def test_simple_diff(self):
        if PY3:
            expected = "'id': 1 != 2"
        else:
            expected = "u'id': 1 != 2"

        self.check_raises(
            SampleModel(id=1), SampleModel(id=2),
            compare=django_compare,
            message=(
                'SampleModel not as expected:\n'
                '\n'
                'same:\n'
                "['value']\n"
                '\n'
                'values differ:\n'+
                expected
            )
        )

    def test_simple_ignore_fields(self):
        django_compare(SampleModel(id=1), SampleModel(id=1),
                       ignore_fields=['id'])

    def test_ignored_because_speshul(self):
        # see http://stackoverflow.com/questions/21925671/convert-django-model-object-to-dict-with-all-of-the-fields-intact
        django_compare(SampleModel(not_editable=1), SampleModel(not_editable=2))

    def test_normal_compare_id_same(self):
        # other diffs ignored
        compare(SampleModel(id=1, value=1), SampleModel(id=1, value=2))

    def test_normal_compare_id_diff(self):
        if PY3:
            expected = (
                "'id': 3 != 4\n"
                "'value': 1 != 2"
            )

        else:
            expected = (
                "'value': 1 != 2\n"
                "u'id': 3 != 4"
            )

        self.check_raises(
            SampleModel(id=3, value=1), SampleModel(id=4, value=2),
            compare=django_compare,
            message=(
                'SampleModel not as expected:\n'
                '\n'
                'values differ:\n'+
                expected
            )
        )



# Copyright (c) 2011-2013 Simplistix Ltd, 2015 Chris Withers
# See license.txt for license details.

# This module contains bits and pieces to achieve compatibility across all the
# versions of python supported.

import doctest
import manuel
import re
import sys
import textwrap

from manuel.codeblock import (
    CODEBLOCK_START,
    CODEBLOCK_END,
    CodeBlock,
    execute_code_block,
    )

# version markers

from ..compat import PY2 as py_2
py_33_plus = sys.version_info[:2] >= (3, 3)
py_34_plus = sys.version_info[:2] >= (3, 4)
py_35_plus = sys.version_info[:2] >= (3, 5)
py_36_plus = sys.version_info[:2] >= (3, 6)

# Python 2.7 compatibility stuff

BYTE_LITERALS = re.compile("^b('.*')$", re.MULTILINE)
UNICODE_LITERALS = re.compile("u('.*')", re.MULTILINE)


def find_code_blocks(document):
    for region in document.find_regions(CODEBLOCK_START, CODEBLOCK_END):
        start_end = CODEBLOCK_START.search(region.source).end()
        source = textwrap.dedent(region.source[start_end:])
        if py_2:
            source = BYTE_LITERALS.sub('\\1', source)
        source = 'from __future__ import print_function\n' + source
        source_location = '%s:%d' % (document.location, region.lineno)
        code = compile(source, source_location, 'exec', 0, True)
        document.claim_region(region)
        region.parsed = CodeBlock(code, source)


class Manuel(manuel.Manuel):
    def __init__(self):
        manuel.Manuel.__init__(self, [find_code_blocks], [execute_code_block])


if py_2:
    class DocTestChecker(doctest.OutputChecker):
        def check_output(self, want, got, optionflags):
            want = BYTE_LITERALS.sub('\\1', want)
            return doctest.OutputChecker.check_output(
                self, want, got, optionflags
                )
else:
    class DocTestChecker(doctest.OutputChecker):
        def check_output(self, want, got, optionflags):
            want = UNICODE_LITERALS.sub('\\1', want)
            return doctest.OutputChecker.check_output(
                self, want, got, optionflags
                )



# Copyright (c) 2008-2014 Simplistix Ltd
# See license.txt for license details.

import os

from doctest import DocTestSuite, ELLIPSIS
from mock import Mock
from tempfile import mkdtemp
from testfixtures import TempDirectory, Replacer, ShouldRaise, compare
from unittest import TestCase, TestSuite, makeSuite

from ..compat import Unicode, PY3
from testfixtures.tests.compat import py_35_plus
from warnings import catch_warnings

from ..rmtree import rmtree

if PY3:
    some_bytes = '\xa3'.encode('utf-8')
    some_text = '\xa3'
else:
    some_bytes = '\xc2\xa3'
    some_text = '\xc2\xa3'.decode('utf-8')


class DemoTempDirectory:

    def test_return_path(self):  # pragma: no branch
        """
        If you want the path created when you use `write`, you
        can do:

        >>> temp_dir.write('filename', b'data')
        '...filename'
        """

    def test_ignore(self):  # pragma: no branch
        """
        TempDirectories can also be set up to ignore certain files:

        >>> d = TempDirectory(ignore=('.svn', ))
        >>> p = d.write('.svn', b'stuff')
        >>> temp_dir.listdir()
        No files or directories found.
        """

    def test_ignore_regex(self):  # pragma: no branch
        """
        TempDirectories can also be set up to ignore certain files:

        >>> d = TempDirectory(ignore=('^\.svn$', '.pyc$'))
        >>> p = d.write('.svn', b'stuff')
        >>> p = d.write('foo.svn', b'')
        >>> p = d.write('foo.pyc', b'')
        >>> p = d.write('bar.pyc', b'')
        >>> d.listdir()
        foo.svn
        """


class TestTempDirectory:

    def test_cleanup(self):  # pragma: no branch
        """
        >>> d = TempDirectory()
        >>> p = d.path
        >>> os.path.exists(p)
        True
        >>> p = d.write('something', b'stuff')
        >>> d.cleanup()
        >>> os.path.exists(p)
        False
        """

    def test_cleanup_all(self):  # pragma: no branch
        """
        If you create several TempDirecories during a doctest,
        or if exceptions occur while running them,
        it can create clutter on disk.
        For this reason, it's recommended to use the classmethod
        TempDirectory.cleanup_all() as a tearDown function
        to remove them all:

        >>> d1 = TempDirectory()
        >>> d2 = TempDirectory()

        Some sanity checks:

        >>> os.path.exists(d1.path)
        True
        >>> p1 = d1.path
        >>> os.path.exists(d2.path)
        True
        >>> p2 = d2.path

        Now we show the function in action:

        >>> TempDirectory.cleanup_all()

        >>> os.path.exists(p1)
        False
        >>> os.path.exists(p2)
        False
        """

    def test_with_statement(self):  # pragma: no branch
        """
        >>> with TempDirectory() as d:
        ...    p = d.path
        ...    print(os.path.exists(p))
        ...    path = d.write('something', b'stuff')
        ...    os.listdir(p)
        ...    with open(os.path.join(p, 'something')) as f:
        ...        print(repr(f.read()))
        True
        ['something']
        'stuff'
        >>> os.path.exists(p)
        False
        """

    def test_listdir_sort(self):  # pragma: no branch
        """
        >>> with TempDirectory() as d:
        ...    p = d.write('ga', b'')
        ...    p = d.write('foo1', b'')
        ...    p = d.write('Foo2', b'')
        ...    p = d.write('g.o', b'')
        ...    d.listdir()
        Foo2
        foo1
        g.o
        ga
        """


class TempDirectoryTests(TestCase):

    def test_write_with_slash_at_start(self):
        with TempDirectory() as d:
            with ShouldRaise(ValueError(
                    'Attempt to read or write outside the temporary Directory'
                    )):
                d.write('/some/folder', 'stuff')

    def test_makedir_with_slash_at_start(self):
        with TempDirectory() as d:
            with ShouldRaise(ValueError(
                    'Attempt to read or write outside the temporary Directory'
                    )):
                d.makedir('/some/folder')

    def test_read_with_slash_at_start(self):
        with TempDirectory() as d:
            with ShouldRaise(ValueError(
                    'Attempt to read or write outside the temporary Directory'
                    )):
                d.read('/some/folder')

    def test_listdir_with_slash_at_start(self):
        with TempDirectory() as d:
            with ShouldRaise(ValueError(
                    'Attempt to read or write outside the temporary Directory'
                    )):
                d.listdir('/some/folder')

    def test_compare_with_slash_at_start(self):
        with TempDirectory() as d:
            with ShouldRaise(ValueError(
                    'Attempt to read or write outside the temporary Directory'
                    )):
                d.compare((), path='/some/folder')

    def test_read_with_slash_at_start_ok(self):
        with TempDirectory() as d:
            path = d.write('foo', b'bar')
            compare(d.read(path), b'bar')

    def test_dont_cleanup_with_path(self):
        d = mkdtemp()
        fp = os.path.join(d, 'test')
        with open(fp, 'w') as f:
            f.write('foo')
        try:
            td = TempDirectory(path=d)
            self.assertEqual(d, td.path)
            td.cleanup()
            # checks
            self.assertEqual(os.listdir(d), ['test'])
            with open(fp) as f:
                self.assertEqual(f.read(), 'foo')
        finally:
            rmtree(d)

    def test_dont_create_with_path(self):
        d = mkdtemp()
        rmtree(d)
        td = TempDirectory(path=d)
        self.assertEqual(d, td.path)
        self.failIf(os.path.exists(d))

    def test_deprecated_check(self):
        with TempDirectory() as d:
            d.write('x', b'')
            d.check('x')

    def test_deprecated_check_dir(self):
        with TempDirectory() as d:
            d.write('foo/x', b'')
            d.check_dir('foo', 'x')

    def test_deprecated_check_all(self):
        with TempDirectory() as d:
            d.write('a/b/c', b'')
            d.check_all('', 'a/', 'a/b/', 'a/b/c')
            d.check_all('a', 'b/', 'b/c')

    def test_compare_sort_actual(self):
        with TempDirectory() as d:
            d.write('ga', b'')
            d.write('foo1', b'')
            d.write('Foo2', b'')
            d.write('g.o', b'')
            d.compare(['Foo2', 'foo1', 'g.o', 'ga'])

    def test_compare_sort_expected(self):
        with TempDirectory() as d:
            d.write('ga', b'')
            d.write('foo1', b'')
            d.write('Foo2', b'')
            d.write('g.o', b'')
            d.compare(['Foo2', 'ga', 'foo1', 'g.o'])

    def test_compare_path_tuple(self):
        with TempDirectory() as d:
            d.write('a/b/c', b'')
            d.compare(path=('a', 'b'),
                      expected=['c'])

    def test_recursive_ignore(self):
        with TempDirectory(ignore=['.svn']) as d:
            d.write('.svn/rubbish', b'')
            d.write('a/.svn/rubbish', b'')
            d.write('a/b/.svn', b'')
            d.write('a/b/c', b'')
            d.write('a/d/.svn/rubbish', b'')
            d.compare([
                'a/',
                'a/b/',
                'a/b/c',
                'a/d/',
                ])

    def test_files_only(self):
        with TempDirectory() as d:
            d.write('a/b/c', b'')
            d.compare(['a/b/c'], files_only=True)

    def test_path(self):
        with TempDirectory() as d:
            expected1 = d.makedir('foo')
            expected2 = d.write('baz/bob', b'')
            expected3 = d.getpath('a/b/c')

            actual1 = d.getpath('foo')
            actual2 = d.getpath('baz/bob')
            actual3 = d.getpath(('a', 'b', 'c'))

        self.assertEqual(expected1, actual1)
        self.assertEqual(expected2, actual2)
        self.assertEqual(expected3, actual3)

    def test_atexit(self):
        # http://bugs.python.org/issue25532
        from mock import call

        m = Mock()
        with Replacer() as r:
            # make sure the marker is false, other tests will
            # probably have set it
            r.replace('testfixtures.TempDirectory.atexit_setup', False)
            r.replace('atexit.register', m.register)

            d = TempDirectory()

            expected = [call.register(d.atexit)]

            compare(expected, m.mock_calls)

            with catch_warnings(record=True) as w:
                d.atexit()
                self.assertTrue(len(w), 1)
                compare(str(w[0].message), (  # pragma: no branch
                    "TempDirectory instances not cleaned up by shutdown:\n" +
                    d.path
                    ))

            d.cleanup()

            compare(set(), TempDirectory.instances)

            # check re-running has no ill effects
            d.atexit()

    def test_read_decode(self):
        with TempDirectory() as d:
            with open(os.path.join(d.path, 'test.file'), 'wb') as f:
                f.write(b'\xc2\xa3')
            compare(d.read('test.file', 'utf8'), some_text)

    def test_read_no_decode(self):
        with TempDirectory() as d:
            with open(os.path.join(d.path, 'test.file'), 'wb') as f:
                f.write(b'\xc2\xa3')
            compare(d.read('test.file'), b'\xc2\xa3')

    def test_write_bytes(self):
        with TempDirectory() as d:
            d.write('test.file', b'\xc2\xa3')
            with open(os.path.join(d.path, 'test.file'), 'rb') as f:
                compare(f.read(), b'\xc2\xa3')

    def test_write_unicode(self):
        with TempDirectory() as d:
            d.write('test.file', some_text, 'utf8')
            with open(os.path.join(d.path, 'test.file'), 'rb') as f:
                compare(f.read(), b'\xc2\xa3')

    def test_write_unicode_bad(self):
        if py_35_plus:
            expected = TypeError(
                "a bytes-like object is required, not 'str'"
                )
        elif PY3:
            expected = TypeError(
                "'str' does not support the buffer interface"
                )
        else:
            expected = UnicodeDecodeError(
                'ascii', '\xa3', 0, 1, 'ordinal not in range(128)'
                )
        with TempDirectory() as d:
            with ShouldRaise(expected):
                d.write('test.file', Unicode('\xa3'))

    def test_just_empty_non_recursive(self):
        with TempDirectory() as d:
            d.makedir('foo/bar')
            d.makedir('foo/baz')
            d.compare(path='foo',
                      expected=['bar', 'baz'],
                      recursive=False)

    def test_just_empty_dirs(self):
        with TempDirectory() as d:
            d.makedir('foo/bar')
            d.makedir('foo/baz')
            d.compare(['foo/', 'foo/bar/', 'foo/baz/'])

    def test_symlink(self):
        with TempDirectory() as d:
            d.write('foo/bar.txt', b'x')
            os.symlink(d.getpath('foo'), d.getpath('baz'))
            d.compare(['baz/', 'foo/', 'foo/bar.txt'])

    def test_follow_symlinks(self):
        with TempDirectory() as d:
            d.write('foo/bar.txt', b'x')
            os.symlink(d.getpath('foo'), d.getpath('baz'))
            d.compare(['baz/', 'baz/bar.txt', 'foo/', 'foo/bar.txt'],
                      followlinks=True)

    def test_trailing_slash(self):
        with TempDirectory() as d:
            d.write('source/foo/bar.txt', b'x')
            d.compare(path='source/', expected=['foo/', 'foo/bar.txt'])

    def test_default_encoding(self):
        encoded = b'\xc2\xa3'
        decoded = encoded.decode('utf-8')
        with TempDirectory(encoding='utf-8') as d:
            d.write('test.txt', decoded)
            compare(d.read('test.txt'), expected=decoded)

    def test_override_default_encoding(self):
        encoded = b'\xc2\xa3'
        decoded = encoded.decode('utf-8')
        with TempDirectory(encoding='ascii') as d:
            d.write('test.txt', decoded, encoding='utf-8')
            compare(d.read('test.txt', encoding='utf-8'), expected=decoded)


# using a set up and teardown function
# gets rid of the need for the imports in
# doc tests


def setUp(test):
    test.globs['temp_dir'] = TempDirectory()


def tearDown(test):
    TempDirectory.cleanup_all()


def test_suite():
    return TestSuite((
        DocTestSuite(setUp=setUp, tearDown=tearDown,
                     optionflags=ELLIPSIS),
        makeSuite(TempDirectoryTests),
        ))
